# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
homepage recommender:ffm
"""
import hashlib
import time


def hashstr(s, nr_bins):
    return str(int(hashlib.md5(s.encode('utf-8')).hexdigest(), 16) % (nr_bins - 1) + 1)


def print_run_time(func):
    def wrapper(*args, **kw):
        local_time = time.time()
        func(*args, **kw)
        print('Current function : {function}, time used : {temps}'.format(
            function=func.__name__, temps=time.time() - local_time))

    return wrapper



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
@version: 0.1
@author: gabriel
@file: sentrys.py
@time: 2018/11/20 15:12
"""
import yaml
from raven import Client


class SentryCli(object):
    def __init__(self, config_file, section="Sentry_dsn"):
        config = yaml.load(open(config_file, 'r'),
                           Loader=yaml.FullLoader)
        try:
            self.client = Client(config.get(section))
        except:
            raise Exception("Sentry_dsn input error!!!")

    def check_commom_err(self, e):
        self.client.captureException(e)



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
@version: 0.1
@author: gabriel
@file: sqlservers.py
@time: 2018/11/20 15:12
"""
import yaml
import pymssql


class SqlserverCli:
    def __init__(self, config_file, section):
        """
        create connection to mssql
        """
        config = yaml.load(open(config_file, 'r'),
                           Loader=yaml.FullLoader)

        try:
            # 打开sqlserver数据库连接
            host = config[section]['Host']
            port = int(config[section]['Port'])
            user = config[section]['User']
            password = config[section]['Password']
            db_name = config[section]['Db_name']
            self.db = pymssql.connect(host=host,
                                      user=user,
                                      password=password,
                                      database=db_name,
                                      port=port,
                                      charset="utf8")
        except Exception as e:
            raise e

    def query_mssql(self, sql):
        """
        query
        """
        with self.db.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def close_mssql(self):
        """
        close connection
        """
        self.db.close()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
homepage recommender:ffm
"""
import datetime
import time


def get_before_date(days=1):
    today = datetime.date.today()
    oneday = datetime.timedelta(days=days)
    yesterday = today - oneday
    return yesterday


def get_after_date(days=1):
    today = datetime.date.today()
    oneday = datetime.timedelta(days=days)
    yesterday = today + oneday
    return yesterday


def getYesterday():
    yesterday = datetime.date.today() + datetime.timedelta(-1)
    return yesterday


def today_time_d():
    return time.strftime("%Y-%m-%d", time.localtime())


def today_time_s():
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


def get_date_from_timstamp(timestamp):
    return datetime.datetime.fromtimestamp(timestamp)


def get_date_from_str(dtstr):
    return datetime.datetime.strptime(dtstr, "%Y-%m-%d")


if __name__ == "__main__":
    print(get_after_date(1))
    print(get_before_date(1))
    print(today_time_s())



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
homepage recommender:ffm
"""
import os


def file_repalce(source, dest):
    os.replace(source, dest)


def get_dir_path(file):
    return os.path.abspath(os.path.dirname(file))


def get_updir_path(file):
    return os.path.dirname(os.path.dirname(os.path.abspath(file)))


def get_upupdir_path(file):
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(file))))



# -*- coding: UTF-8 -*-
"""
@version: 0.1
@author: zhanglanhui
@file: hdfs.py
@time: 2019/07/08
"""
import pyhdfs


# 关于python操作hdfs的API可以查看官网:
# https://hdfscli.readthedocs.io/en/latest/api.html
class HDFSCli(object):
    def __init__(self, host, user='root'):
        self.fs = pyhdfs.HdfsClient(hosts=host, user_name=user)

    # 创建目录
    def mkdirs(self, hdfs_path):
        self.fs.mkdirs(hdfs_path)

    # 删除hdfs文件
    def delete_hdfs_file(self, hdfs_path):
        self.fs.delete(hdfs_path)

    # 判断文档存在
    def is_exist_file(self, hdfs_path) -> bool:
        return self.fs.exists(hdfs_path)

    # 移动或者修改文件
    def move_or_rename(self, hdfs_src_path, hdfs_dst_path) -> bool:
        return self.fs.rename(hdfs_src_path, hdfs_dst_path)

    # 从hdfs获取文件到本地
    def get_from_hdfs(self, local_path, hdfs_path):
        self.fs.copy_to_local(hdfs_path, local_path)

    # 上传文件到hdfs
    def put_to_hdfs(self, local_path, hdfs_path):
        self.fs.copy_from_local(local_path, hdfs_path)

    # 追加数据到hdfs文件
    def append_to_hdfs(self, hdfs_path, data):
        self.fs.append(hdfs_path, data, overwrite=False, append=True)

    # 覆盖数据写到hdfs文件
    def write_to_hdfs(self, local_file, hdfs_path):
        if self.is_exist_file(hdfs_path):
            self.delete_hdfs_file(hdfs_path)
            self.put_to_hdfs(local_file, hdfs_path)
        else:
            self.put_to_hdfs(local_file, hdfs_path)

    # 返回目录下的文件
    def list(self, hdfs_path):
        return self.fs.listdir(hdfs_path)


if __name__ == '__main__':
    fs = HDFSCli(host='test:50070', user='root')
    print(fs.list('/test'))



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
homepage recommender:ffm
"""
import yaml


def read_from_yaml(file: str) -> dict:
    conf_dic = yaml.load(open(file, 'r'),
                         Loader=yaml.FullLoader)

    return conf_dic


if __name__ == "__main__":
    print(read_from_yaml("config/dev/server_config.yaml"))



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
hive
"""
import yaml
from pyhive import hive


class HiveCli:
    def __init__(self, config_file, section, authMechanism="KERBEROS"):
        """
        create connection to hive server2
        """
        config = yaml.load(open(config_file, 'r'),
                           Loader=yaml.FullLoader)

        try:
            host = config[section]['Host']
            port = int(config[section]['Port'])
            db_name = config[section]['Db_name']
            self.conn = hive.connect(host=host,
                                     port=port,
                                     auth=authMechanism,
                                     kerberos_service_name="hive",
                                     database=db_name)
        except Exception as e:
            raise e

    def query_hive(self, sql):
        """
        query
        """
        with self.conn.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def close_hive(self):
        """
        close connection
        """
        self.conn.close()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
@version: 0.1
@author: gabriel
@file: mysqls.py
@time: 2018/11/20 15:12
"""
import yaml
import math
import pymysql.cursors


class MysqlCli:
    def __init__(self, config_file, section):
        """
        create connection to mysql
        """
        config = yaml.load(open(config_file, 'r'),
                           Loader=yaml.FullLoader)

        try:
            host = config[section]['Host']
            port = int(config[section]['Port'])
            user = config[section]['User']
            password = config[section]['Password']
            db_name = config[section]['Db_name']
            self.table_name = config[section]['Table_name']
            self.db_info = pymysql.connect(host=host,
                                           user=user,
                                           passwd=password,
                                           db=db_name,
                                           port=port,
                                           charset="utf8")
        except Exception as e:
            raise e

    def table_name(self):
        """
        table dict
        """
        return self.table_name

    def query_mysql(self, sql):
        """
        query
        """
        with self.db_info.cursor() as cursor:
            cursor.execute(sql)
            self.db_info.commit()
            return cursor.fetchall()

    def execute_mysql(self, sql):
        """
        query
        """
        with self.db_info.cursor() as cursor:
            cursor.execute(sql)
            self.db_info.commit()

    def cursor_mysql(self):
        return self.db_info.cursor()

    def ping_mysql(self):
        self.db_info.ping(True)

    def update_data_by_insert(self, features: dict, key: str, field: str, table: str, times=1000):
        if len(features) < 1:
            return

        feature_list = [[k, v] for k, v in features.items()]
        try:
            L = int(math.ceil(len(features) / times))
            for d in [feature_list[i:i + L] for i in range(0, len(feature_list), L)]:
                sql = """
                INSERT INTO 
                `{t}` (`{k}`,`{f}`) 
                VALUES {d} on DUPLICATE key 
                update `{f}`=values(`{f}`);
                """.format(t=table,
                           d=','.join(["('{}','{}')".format(x[0], x[1]) for x in d]),
                           k=key,
                           f=field)
                self.execute_mysql(sql)
        except Exception as e:
            self.db_info.rollback()
            raise e

    def commit(self):
        self.db_info.commit()

    def rollback(self):
        self.db_info.rollback()

    def close_mysql(self):
        """
        close connection
        """
        self.db_info.close()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zhanglanhui
homepage recommender:ffm
"""
import copy


def _list_merge(l1, l2):
    # 将所有的漫画按照type分类
    tmp = copy.deepcopy(l1)
    tmp.extend([x for x in l2 if x not in l1])
    return tmp


def check_length(l, length):
    if len(l) < length:
        raise ValueError("len of list is less then {}".format(length))
